@@ -19,12 +19,15 @@
 
 #include "omalloc/omalloc.h"
 #include "misc/options.h"
-
 #include "polys/monomials/p_polys.h"
 #include "polys/kbuckets.h"
 
 #include "kernel/polys.h"
 
+#ifdef HAVE_SHIFTBBA
+#include "polys/shiftop.h"
+#endif
+
 
 #define HAVE_TAIL_BIN
 // This doesn't really work, fixme, if necessary
@@ -119,12 +122,32 @@ KINLINE void sTObject::Set(poly p_in, ring r)
   if (r != currRing)
   {
     assume(r == tailRing);
-    p_Test(p_in, r);
+#ifdef HAVE_SHIFTBBA
+    if (r->isLPring)
+    {
+      shift = si_max(p_mFirstVblock(p_in, r) - 1, 0);
+      if (!shift) p_Test(p_in, r);
+    }
+    else
+#endif
+    {
+      p_Test(p_in, r);
+    }
     t_p = p_in;
   }
   else
   {
-    p_Test(p_in, currRing);
+#ifdef HAVE_SHIFTBBA
+    if (currRing->isLPring)
+    {
+      shift = si_max(p_mFirstVblock(p_in, currRing) - 1, 0);
+      if (!shift) p_Test(p_in, currRing);
+    }
+    else
+#endif
+    {
+      p_Test(p_in, currRing);
+    }
     p = p_in;
   }
   pLength=::pLength(p_in);
@@ -141,7 +164,17 @@ KINLINE void sTObject::Set(poly p_in, ring c_r, ring t_r)
   if (c_r != t_r)
   {
     assume(c_r == currRing && t_r == tailRing);
-    p_Test(p_in, currRing);
+#ifdef HAVE_SHIFTBBA
+    if (c_r->isLPring)
+    {
+      shift = si_max(p_mFirstVblock(p_in, c_r) - 1, 0);
+      if (!shift) p_Test(p_in, currRing);
+    }
+    else
+#endif
+    {
+      p_Test(p_in, currRing);
+    }
     p = p_in;
     pLength=::pLength(p_in);
   }
@@ -206,8 +239,7 @@ KINLINE void sTObject::Copy()
     t_p = p_Copy(t_p, tailRing);
     if (p != NULL) /* and t_p!=NULL*/
     {
-      p = p_Head(p, currRing);
-      n_Delete(&pGetCoeff(p),currRing->cf);
+      p = p_LmInit(p, currRing);
       pGetCoeff(p)=pGetCoeff(t_p);
       pNext(p) = pNext(t_p);
     }
@@ -334,6 +366,13 @@ KINLINE void sLObject::Normalize()
   {
     pNormalize(p);
   }
+  if (bucket!=NULL) kBucketNormalize(bucket);
+}
+
+KINLINE void sLObject::CanonicalizeP()
+{
+  if (bucket != NULL)
+    kBucketCanonicalize(bucket);
 }
 
 KINLINE void sLObject::HeadNormalize()
@@ -640,21 +679,6 @@ KINLINE poly sLObject::LmExtractAndIter()
   return ret;
 }
 
-KINLINE poly sLObject::CanonicalizeP()
-{
-  //kTest_L(this);
-  int i = -1;
-
-  if (bucket != NULL)
-    i = kBucketCanonicalize(bucket);
-
-  if (p == NULL)
-    p = k_LmInit_tailRing_2_currRing(t_p, tailRing);
-
-  if (i >= 0) pNext(p) = bucket->buckets[i];
-  return p;
-}
-
 KINLINE poly sLObject::GetTP()
 {
   //kTest_L(this);
@@ -758,7 +782,7 @@ KINLINE long sLObject::pLDeg(BOOLEAN deg_last)
 
   long ldeg;
   ldeg = tailRing->pLDeg(GetLmTailRing(), &length, tailRing);
-#ifdef HAVE_ASSUME
+#ifndef SING_NDEBUG
   if ( pLength == 0)
     p_Last(GetLmTailRing(), pLength, tailRing);
   assume ( pLength == length || rIsSyzIndexRing(currRing));
@@ -1028,6 +1052,44 @@ KINLINE void k_GetStrongLeadTerms(const poly p1, const poly p2, const ring leadR
  * Misc things
  *
  ***************************************************************/
+KINLINE int ksReducePolyTailLC_Z(LObject* PR, TObject* PW, LObject* Red)
+{
+  BOOLEAN ret;
+  number mult, rest;
+  TObject red = *PW;
+  red.Copy();
+  rest = n_QuotRem(pGetCoeff(Red->p), pGetCoeff(red.p),
+          &mult, currRing->cf);
+  red.Mult_nn(rest);
+
+  assume(PR->GetLmCurrRing() != red.GetLmCurrRing());
+  ret = ksReducePolyLC(Red, &red, NULL, &mult);
+  red.Delete();
+  red.Clear();
+
+  return ret;
+}
+
+KINLINE int ksReducePolyTail_Z(LObject* PR, TObject* PW, LObject* Red)
+{
+  BOOLEAN ret;
+  number coef;
+
+  assume(PR->GetLmCurrRing() != PW->GetLmCurrRing());
+  ret = ksReducePoly(Red, PW, NULL, &coef);
+
+  if (!ret)
+  {
+    if (! n_IsOne(coef, currRing->cf))
+    {
+      PR->Mult_nn(coef);
+      // HANNES: mark for Normalize
+    }
+    n_Delete(&coef, currRing->cf);
+  }
+  return ret;
+}
+
 KINLINE int ksReducePolyTail(LObject* PR, TObject* PW, LObject* Red)
 {
   BOOLEAN ret;
