@@ -15,10 +15,10 @@
 #define HAVE_SVD
 
 #undef VERSION
-#define VERSION "4.1.1"
+#define VERSION "4.1.2"
 #define SINGULAR_MAJOR_VERSION 4
 #define SINGULAR_MINOR_VERSION 1
-#define SINGULAR_SUB_VERSION 1
+#define SINGULAR_SUB_VERSION 2
 #define S_ROOT_DIR ""
 
 /*******************************************************************
@@ -52,14 +52,6 @@
 /* linear algebra extensions from pcv.h/pcv.cc */
 #define HAVE_PCV 1
 
-/* procedures to compute groebner bases with the f5 implementation */
-/* still testing */
-#undef HAVE_F5
-
-/* procedures to compute groebner bases with the f5c implementation */
-/* still testing */
-#undef HAVE_F5C
-
 /* procedures to compute with units */
 #define HAVE_UNITS
 
@@ -84,7 +76,7 @@
 #define ALIGN_8
 #endif
 
-#define SINGULAR_PATCHLEVEL 0
+#define SINGULAR_PATCHLEVEL 4
 #define SINGULAR_VERSION ((SINGULAR_MAJOR_VERSION*1000 + SINGULAR_MINOR_VERSION*100 + SINGULAR_SUB_VERSION*10)+SINGULAR_PATCHLEVEL)
 
 /*******************************************************************
@@ -133,17 +125,6 @@ static inline void HALT() { m2_end(2);}
 /* #define MAX_INT_LEN 11 */
 
 
-#ifdef DO_PROFILE
-/* define to enable explicit profiling of some crucial inline
- * routines and defines  */
-#undef DO_DEEP_PROFILE
-#endif
-
-/* define to enable assume */
-#ifndef HAVE_ASSUME
-#undef HAVE_ASSUME
-#endif
-
 /* define LINKAGE to "extern C" if compiling for shared libs */
 #ifndef LINKAGE
 #if defined(PIC)
@@ -164,9 +145,6 @@ static inline void HALT() { m2_end(2);}
 /* undefine to enable inline */
 #define NO_INLINE
 
-/* undefine to disable assume -- should normally be defined for SING_NDEBUG */
-#define HAVE_ASSUME
-
 /* undef PDEBUG to disable checks of polys
 
  define PDEBUG to
@@ -393,10 +371,10 @@ extern void dErrorBreak();
 }
 #endif
 
-#ifndef HAVE_ASSUME
+#ifdef SING_NDEBUG
 #define assume(x) do {} while (0)
 #define r_assume(x) do {} while (0)
-#else /* ! HAVE_ASSUME */
+#else /* SING_NDEBUG */
 
 #define assume_violation(s,f,l) \
   dReportError("assume violation at %s:%d condition: %s", f,l,s)
@@ -424,10 +402,10 @@ do                                              \
   }                                             \
 }                                               \
 while (0)
-#endif /* HAVE_ASSUME */
+#endif /* SING_NDEBUG */
 
 /* do have RDEBUG, unless we are doing the very real thing */
-#ifdef HAVE_ASSUME
+#ifndef SING_NDEBUG
 #ifndef RDEBUG
 #define RDEBUG
 #endif
