@@ -8,27 +8,19 @@
 */
 
 /* includes */
-#include <omalloc/omalloc.h>
-#include <misc/auxiliary.h>
-#include <coeffs/coeffs.h>
-//#include <polys/monomials/polys-impl.h>
+#include "misc/auxiliary.h"
+#include "coeffs/coeffs.h"
+#include "misc/intvec.h"
+#include "misc/int64vec.h"
+#include "coeffs/coeffs.h" // ring,number
+#include "polys/monomials/monomials.h"
+//#include "polys/monomials/polys-impl.h"
 //
 
 /* forward declaration of types */
 class idrec; typedef idrec *   idhdl; // _only_ for idhdl ip_sring::idroot
-struct  spolyrec;
-typedef struct spolyrec    polyrec;
-typedef struct spolyrec *         poly;
-typedef struct spolyrec const *   const_poly;
-struct ip_sring;
-typedef struct ip_sring *         ring;
-typedef struct ip_sring const *   const_ring;
-class intvec;
-class int64vec;
 struct p_Procs_s;
 typedef struct p_Procs_s p_Procs_s;
-//class slists;
-//typedef slists *           lists;
 class kBucket;
 typedef kBucket*           kBucket_pt;
 
@@ -319,6 +311,7 @@ struct ip_sring
 
 #ifdef HAVE_SHIFTBBA
   short          isLPring; /* 0 for non-letterplace rings, otherwise the number of LP blocks, at least 1, known also as lV */
+  short          LPncGenCount;
 #endif
 
   BOOLEAN   VectorOut;
@@ -388,7 +381,8 @@ struct ip_sring
 ring   rDefault(int ch, int N, char **n);
 ring   rDefault(const coeffs cf, int N, char **n, const rRingOrder_t o=ringorder_lp);
 ring   rDefault(int ch, int N, char **n,int ord_size, rRingOrder_t *ord, int *block0, int *block1, int **wvhdl=NULL);
-ring   rDefault(const coeffs cf, int N, char **n,int ord_size, rRingOrder_t *ord, int *block0, int *block1, int **wvhdl=NULL);
+ring   rDefault(const coeffs cf, int N, char **n,int ord_size, rRingOrder_t *ord, int *block0, int *block1, int **wvhdl=NULL, unsigned long bitmask=0);
+unsigned long rGetExpSize(unsigned long bitmask, int & bits, int N);
 
 // #define rIsRingVar(A) r_IsRingVar(A,currRing)
 int    r_IsRingVar(const char *n, char**names, int N);
@@ -403,10 +397,29 @@ ring   rEnvelope(ring r);
 /// we must always have this test!
 static inline BOOLEAN rIsPluralRing(const ring r)
 {
-  assume(r != NULL); assume(r->cf != NULL);
+  assume(r != NULL);
 #ifdef HAVE_PLURAL
   nc_struct *n;
-  return (r != NULL) && ((n=r->GetNC()) != NULL) /*&& (n->type != nc_error)*/;
+  return ((n=r->GetNC()) != NULL) /*&& (n->type != nc_error)*/;
+#else
+  return FALSE;
+#endif
+}
+
+static inline BOOLEAN rIsLPRing(const ring r)
+{
+  assume(r != NULL);
+#if defined(HAVE_PLURAL) && defined(HAVE_SHIFTBBA)
+  return (r->isLPring!=0);
+#else
+  return FALSE;
+#endif
+}
+
+static inline BOOLEAN rIsNCRing(const ring r)
+{
+#ifdef HAVE_PLURAL
+  return rIsLPRing(r) || rIsPluralRing(r);
 #else
   return FALSE;
 #endif
@@ -458,22 +471,19 @@ BOOLEAN rSamePolyRep(ring r1, ring r2);
 
 void   rUnComplete(ring r);
 
-BOOLEAN rRing_is_Homog(ring r);
-BOOLEAN rRing_has_CompLastBlock(ring r);
+BOOLEAN rRing_is_Homog(const ring r);
+BOOLEAN rRing_has_CompLastBlock(const ring r);
+BOOLEAN rRing_ord_pure_dp(const ring r);
+BOOLEAN rRing_ord_pure_Dp(const ring r);
+BOOLEAN rRing_ord_pure_lp(const ring r);
 
 #ifdef HAVE_RINGS
 static inline BOOLEAN rField_is_Ring_2toM(const ring r)
 { assume(r != NULL); assume(r->cf != NULL); return ( nCoeff_is_Ring_2toM(r->cf) ); }
 
-static inline BOOLEAN rField_is_Ring_ModN(const ring r)
-{ assume(r != NULL); assume(r->cf != NULL); return ( nCoeff_is_Ring_ModN(r->cf) ); }
-
 static inline BOOLEAN rField_is_Ring_PtoM(const ring r)
 { assume(r != NULL); assume(r->cf != NULL); return ( nCoeff_is_Ring_PtoM(r->cf) ); }
 
-static inline BOOLEAN rField_is_Ring_Z(const ring r)
-{ assume(r != NULL); assume(r->cf != NULL); return ( nCoeff_is_Ring_Z(r->cf) ); }
-
 static inline BOOLEAN rField_is_Ring(const ring r)
 { assume(r != NULL); assume(r->cf != NULL); return nCoeff_is_Ring(r->cf); }
 
@@ -485,9 +495,7 @@ static inline BOOLEAN rField_has_Units(const ring r)
 #else
 #define rField_is_Ring(A) (0)
 #define rField_is_Ring_2toM(A) (0)
-#define rField_is_Ring_ModN(A) (0)
 #define rField_is_Ring_PtoM(A) (0)
-#define rField_is_Ring_Z(A) (0)
 #define rField_is_Domain(A) (1)
 #define rField_has_Units(A) (1)
 #endif
@@ -504,6 +512,9 @@ static inline BOOLEAN rField_is_Q(const ring r)
 static inline BOOLEAN rField_is_Z(const ring r)
 { assume(r != NULL); assume(r->cf != NULL); return nCoeff_is_Z(r->cf); }
 
+static inline BOOLEAN rField_is_Zn(const ring r)
+{ assume(r != NULL); assume(r->cf != NULL); return nCoeff_is_Zn(r->cf); }
+
 static inline BOOLEAN rField_is_numeric(const ring r) /* R, long R, long C */
 { assume(r != NULL); assume(r->cf != NULL); return nCoeff_is_numeric(r->cf); }
 
@@ -544,7 +555,8 @@ static inline BOOLEAN rField_has_simple_inverse(const ring r)
 static inline BOOLEAN rField_has_simple_Alloc(const ring r)
 { assume(r != NULL); assume(r->cf != NULL); return nCoeff_has_simple_Alloc(r->cf); }
 
-n_coeffType rFieldType(const ring r);
+/// the type of the coefficient filed of r (n_Zp, n_Q, etc)
+static inline n_coeffType rFieldType(const ring r) { return (r->cf->type); }
 
 /// this needs to be called whenever a new ring is created: new fields
 /// in ring are created (like VarOffset), unless they already exist
@@ -731,8 +743,8 @@ ring   rAssure_c_dp(const ring r);
 ring   rAssure_CompLastBlock(const ring r, BOOLEAN complete = TRUE);
 
 /// makes sure that c/C ordering is last ordering and SyzIndex is first
-ring   rAssure_SyzComp_CompLastBlock(const ring r, BOOLEAN complete = TRUE);
-ring   rAssure_TDeg(const ring r, int start_var, int end_var, int &pos);
+ring   rAssure_SyzComp_CompLastBlock(const ring r);
+ring   rAssure_TDeg(const ring r, int &pos);
 
 /// return the max-comonent wchich has syzIndex i
 /// Assume: i<= syzIndex_limit
@@ -754,9 +766,6 @@ inline BOOLEAN rHasMixedOrdering(const ring r) { return (r->MixedOrder); }
 // #define rHasGlobalOrdering(R) ((R)->OrdSgn==1)
 // #define rHasLocalOrMixedOrdering(R) ((R)->OrdSgn==-1)
 
-#define rHasGlobalOrdering_currRing() rHasGlobalOrdering(currRing)
-#define rHasLocalOrMixedOrdering_currRing() rHasLocalOrMixedOrdering(currRing)
-
 BOOLEAN rOrd_is_Totaldegree_Ordering(const ring r);
 
 /// return TRUE if p_SetComp requires p_Setm
@@ -795,7 +804,6 @@ void rKillModifiedRing(ring r);
 void rKillModified_Wp_Ring(ring r);
 
 ring rModifyRing_Simple(ring r, BOOLEAN omit_degree, BOOLEAN omit_comp, unsigned long exp_limit, BOOLEAN &simple);
-void rKillModifiedRing_Simple(ring r);
 
 #ifdef RDEBUG
 void rDebugPrint(const ring r);
@@ -825,5 +833,12 @@ int rTypeOfMatrixOrder(const intvec *order);
 
 void rDelete(ring r); // To be used instead of rKill!
 
-extern omBin sip_sring_bin;
+EXTERN_VAR omBin sip_sring_bin;
+
+// ring manipulation
+/// K[x],"y" -> K[x,y] resp. K[y,x]
+ring rPlusVar(const ring r, char *v,int left);
+
+/// undo rPlusVar
+ring rMinusVar(const ring r, char *v);
 #endif
